package org.firstinspires.ftc.teamcode;

import com.acmerobotics.dashboard.FtcDashboard;
import com.qualcomm.robotcore.eventloop.opmode.Autonomous;
import com.qualcomm.robotcore.eventloop.opmode.LinearOpMode;
import com.qualcomm.robotcore.hardware.ColorSensor;
import com.qualcomm.robotcore.hardware.DcMotor;
import com.qualcomm.robotcore.hardware.DistanceSensor;
import com.qualcomm.robotcore.hardware.TouchSensor;

import org.firstinspires.ftc.robotcore.external.hardware.camera.WebcamName;
import org.firstinspires.ftc.robotcore.external.navigation.DistanceUnit;
import org.firstinspires.ftc.teamcode.cometbots.MovementLibrary;
import org.openftc.easyopencv.OpenCvCamera;
import org.openftc.easyopencv.OpenCvCameraFactory;
import org.openftc.easyopencv.OpenCvCameraRotation;

@Autonomous(name = "CometBot Sensor Demo", group = "[14493] CometBots")
public class CometBotsSensorDemo extends LinearOpMode {

    private OpenCvCamera controlHubCam;  // Use OpenCvCamera class from FTC SDK
    private static final int CAMERA_WIDTH = 1280; // width  of wanted camera resolution
    private static final int CAMERA_HEIGHT = 720; // height of wanted camera resolution

    // Calculate the distance using the formula
    public static final double objectWidthInRealWorldUnits = 3.75;  // Replace with the actual width of the object in real-world units
    public static final double focalLength = 728;  // Replace with the focal length of the camera in pixels

    private DcMotor rhw = null;
    private DcMotor rlw = null;
    private DcMotor lhw = null;
    private DcMotor llw = null;

    @Override
    public void runOpMode() throws InterruptedException {

        // Initialize the hardware variables. Note that the strings used here must correspond
        // to the names assigned during the robot configuration step on the DS or RC devices.
        rhw = hardwareMap.get(DcMotor.class, "right hand wheel");
        rlw = hardwareMap.get(DcMotor.class, "right leg wheel");
        lhw = hardwareMap.get(DcMotor.class, "left hand wheel");
        llw = hardwareMap.get(DcMotor.class, "left leg wheel");

        /*
            Put all motors in an array for easier handling

            NOTE: Placement of motor in array is important especially when dealing with functions
            that set direction
        */
        DcMotor[] motors = {rhw, rlw, lhw, llw};

        // Create an instance of the camera
        int cameraMonitorViewId = hardwareMap.appContext.getResources().getIdentifier(
                "cameraMonitorViewId", "id", hardwareMap.appContext.getPackageName());

        // Use OpenCvCameraFactory class from FTC SDK to create camera instance
        controlHubCam = OpenCvCameraFactory.getInstance().createWebcam(
                hardwareMap.get(WebcamName.class, "watch out"), cameraMonitorViewId);

        TouchSensor touchSensor = hardwareMap.get(TouchSensor.class, "touchsensor");
        DistanceSensor distanceSensor = hardwareMap.get(DistanceSensor.class, "distancesensor");
        ColorSensor colorSensor = hardwareMap.get(ColorSensor.class, "colorsensor");

        controlHubCam.openCameraDevice();
        controlHubCam.startStreaming(CAMERA_WIDTH, CAMERA_HEIGHT, OpenCvCameraRotation.UPRIGHT);

        FtcDashboard.getInstance().startCameraStream(controlHubCam, 0);

        /*
            Wait for the driver to hit START in the driver hub
         */
        waitForStart();

        while (opModeIsActive()) {

            /*
                Capture color readout from color sensor
            */
            telemetry.addData("Color Sensor [Red]", colorSensor.red());
            telemetry.addData("Color Sensor [Green]", colorSensor.green());
            telemetry.addData("Color Sensor [Blue]", colorSensor.blue());
            telemetry.addData("Color Sensor [Alpha]", colorSensor.alpha());

            /*
                Capture distance readout from distance sensor
            */
            telemetry.addData("Distance Sensor [inches]", distanceSensor.getDistance(DistanceUnit.INCH));

            /*
                Capture state when touch sensor is pressed
            */
            telemetry.addData("Touch sensor pressed?", touchSensor.isPressed());
            if (touchSensor.isPressed()) {
                telemetry.speak("Touch sensor is pressed");
            }

            if (distanceSensor.getDistance(DistanceUnit.INCH) < 3) {
                telemetry.speak("Warning, unknown object near robot");
                MovementLibrary.setDirectionRight(motors);
                MovementLibrary.setSpeed(motors,0.25);
                sleep(1500);
                MovementLibrary.setSpeed(motors,0);
            }

            /*
                Update stats continually
             */
            telemetry.update();
        }
    }
}
